/*********************************************************************
*                SEGGER MICROCONTROLLER GmbH & Co. KG                *
*        Solutions for real time microcontroller applications        *
**********************************************************************
*                                                                    *
*        (c) 2003-2014     SEGGER Microcontroller GmbH & Co KG       *
*                                                                    *
*        Internet: www.segger.com    Support:  support@segger.com    *
*                                                                    *
**********************************************************************

----------------------------------------------------------------------
File    : USB_MTP.h
Purpose : Public header of USB MTP (Media Transfer Protocol)
--------  END-OF-HEADER  ---------------------------------------------
*/

#ifndef MTP_H          /* Avoid multiple inclusion */
#define MTP_H

#include "SEGGER.h"

#if defined(__cplusplus)
extern "C" {     /* Make sure we have C-declarations in C++ programs */
#endif

/*********************************************************************
*
*       defines, configurable
*
**********************************************************************
*/
#ifndef   MTP_DEBUG_LEVEL
  #define MTP_DEBUG_LEVEL         0       // Debugging is disabled by default
#endif

#ifndef   MTP_USB_CLASS
  #define MTP_USB_CLASS           6       // Class of USB device
#endif

#ifndef   MTP_USB_SUBCLASS
  #define MTP_USB_SUBCLASS        1       // Subclass of USB device
#endif

#ifndef   MTP_USB_PROTOCOL
  #define MTP_USB_PROTOCOL        1       // Protocol of USB device
#endif

#ifndef   MTP_MAX_MUM_STORAGES
  #define MTP_MAX_MUM_STORAGES    4       // Maximum number of storage units
#endif

#ifndef   MTP_SAVE_FILE_INFO
  #define MTP_SAVE_FILE_INFO      0       // Information related to file is stored in object instance. Requires more RAM.
#endif

#ifndef   MTP_MAX_FILE_PATH
  #define MTP_MAX_FILE_PATH       256     // Maximum number of characters in the path of a file or directory
#endif

#ifndef   MTP_SUPPORT_UTF8
  #define MTP_SUPPORT_UTF8        1       // File/directory names are encoded in UTF-8 format
#endif

/*********************************************************************
*
*       defines, non-configurable
*
**********************************************************************
*/
#define MTP_NUM_BYTES_FILE_ID     16

#define MTP_FILE_ATTR_WP          0x01
#define MTP_FILE_ATTR_SYSTEM      0x02
#define MTP_FILE_ATTR_HIDDEN      0x04

/*********************************************************************
*
*       USB_MTP_INST_DATA_DRIVER
*/
typedef struct USB_MTP_INST_DATA_DRIVER {
  const char * pRootDir;          // Directory to be used as root of the storage
} USB_MTP_INST_DATA_DRIVER;

/*********************************************************************
*
*       USB_MTP_STORAGE_INFO
*/
typedef struct USB_MTP_STORAGE_INFO {
  U32  NumKBytesTotal;            // Storage capacity in kBytes
  U32  NumKBytesFreeSpace;        // Available free space on storage in kBytes
  U16  FSType;                    // Type of file system as specified by MTP
  U8   IsWriteProtected;          // Set to 1 if the storage medium can not be modified
  U8   IsRemovable;               // Set to 1 if the storage medium can be removed from device
  char DirDelimiter;              // Character which separates the directory/file names in a path
} USB_MTP_STORAGE_INFO;

/*********************************************************************
*
*       USB_MTP_FILE_INFO
*/
typedef struct USB_MTP_FILE_INFO {
  char * pFilePath;               // Full path to file
  char * pFileName;               // Pointer to beginning of file/directory name in pFilePath
  U32    FileSize;                // Size of the file in bytes. 0xFFFFFFFF when larger than 4GB.
  U32    CreationTime;            // The time when the file was created
  U32    LastWriteTime;           // The time when the file was last modified
  U8     IsDirectory;             // Set to 1 if the path points to a directory
  U8     Attributes;              // Bitmask of file attributes (MTP_FILE_ATTR_...)
  U8     acId[MTP_NUM_BYTES_FILE_ID]; // Unique identifier which persists between MTP sessions
} USB_MTP_FILE_INFO;

/*********************************************************************
*
*       USB_MTP_STORAGE_API
*/
typedef struct USB_MTP_STORAGE_API {
  void       (*pfInit)                (U8 Unit, const USB_MTP_INST_DATA_DRIVER * pDriverData);
  void       (*pfGetInfo)             (U8 Unit, USB_MTP_STORAGE_INFO * pStorageInfo);
  int        (*pfFindFirstFile)       (U8 Unit, const char * pDirPath, USB_MTP_FILE_INFO * pFileInfo);
  int        (*pfFindNextFile)        (U8 Unit, USB_MTP_FILE_INFO * pFileInfo);
  int        (*pfOpenFile)            (U8 Unit, const char * pFilePath);
  int        (*pfCreateFile)          (U8 Unit, const char * pDirPath, USB_MTP_FILE_INFO * pFileInfo);
  int        (*pfReadFromFile)        (U8 Unit, U32 Off, void * pData, U32 NumBytes);
  int        (*pfWriteToFile)         (U8 Unit, U32 Off, const void * pData, U32 NumBytes);
  int        (*pfCloseFile)           (U8 Unit);
  int        (*pfRemoveFile)          (U8 Unit, const char * pFilePath);
  int        (*pfCreateDir)           (U8 Unit, const char * pDirPath, USB_MTP_FILE_INFO * pFileInfo);
  int        (*pfRemoveDir)           (U8 Unit, const char * pDirPath);
  int        (*pfFormat)              (U8 Unit);
  int        (*pfRenameFile)          (U8 Unit, USB_MTP_FILE_INFO * pFileInfo);
  void       (*pfDeInit)              (U8 Unit);
  int        (*pfGetFileAttributes)   (U8 Unit, const char * pFilePath, U8 * pMask);
  int        (*pfModifyFileAttributes)(U8 Unit, const char * pFilePath, U8 SetMask, U8 ClrMask);
  int        (*pfGetFileCreationTime) (U8 Unit, const char * pFilePath, U32 * pTime);
  int        (*pfGetFileLastWriteTime)(U8 Unit, const char * pFilePath, U32 * pTime);
  int        (*pfGetFileId)           (U8 Unit, const char * pFilePath, U8  * pId);
  int        (*pfGetFileSize)         (U8 Unit, const char * pFilePath, U32 * pFileSize);
} USB_MTP_STORAGE_API;

/*********************************************************************
*
*       USB_MTP_INST_DATA
*/
typedef struct USB_MTP_INST_DATA {
  const USB_MTP_STORAGE_API * pAPI;              // Storage driver
  const char                * sDescription;      // Human-readable string which identifies the storage
  const char                * sVolumeId;         // Unique volume identifier
  USB_MTP_INST_DATA_DRIVER    DriverData;        // Config data to be passed to storage driver
} USB_MTP_INST_DATA;

/*********************************************************************
*
*       USB_MTP_INIT_DATA
*/
typedef struct USB_MTP_INIT_DATA {
  U8     EPIn;                // Device to host (bulk)
  U8     EPOut;               // Host to device (bulk)
  U8     EPInt;               // Device to host (interrupt)
  void * pObjectList;         // Memory for the object list
  U32    NumBytesObjectList;  // Number of bytes allocated for the object list
  void * pDataBuffer;         // Transaction send/receive buffer
  U32    NumBytesDataBuffer;  // Number of bytes in the send/receive buffer.
  //
  // The following fields are used internally by the MTP component.
  //
  U8     InterfaceNum;
  U32    NumBytesAllocated;
  U32    NumObjects;
} USB_MTP_INIT_DATA;

/*********************************************************************
*
*       API functions
*
**********************************************************************
*/
void USB_MTP_Init      (void);
int  USB_MTP_Add       (const USB_MTP_INIT_DATA * pInitData);
int  USB_MTP_AddStorage(const USB_MTP_INST_DATA * pInstData);
void USB_MTP_Task      (void);

/*********************************************************************
*
*       Functions defined in the application
*
**********************************************************************
*/
const char * USB_MTP_GetManufacturer (void);
const char * USB_MTP_GetModel        (void);
const char * USB_MTP_GetDeviceVersion(void);
const char * USB_MTP_GetSerialNumber (void);

/*********************************************************************
*
*       Storage drivers
*
**********************************************************************
*/
extern const USB_MTP_STORAGE_API USB_MTP_StorageFS;

#if defined(__cplusplus)
  }              /* Make sure we have C-declarations in C++ programs */
#endif

#endif                 /* Avoid multiple inclusion */

/**************************** end of file ***************************/

